﻿using InternetShopContracts.DataBindingModels;
using InternetShopContracts.DataSearchModels;
using InternetShopContracts.DataViewModels;
using InternetShopContracts.LogicsContracts;

namespace InternetShopForms.Products
{
    public partial class FormProductsList : Form
    {
        private IProductLogic _productLogic;
        private List<ProductViewModel> _products;
        public FormProductsList(IProductLogic productLogic)
        {
            InitializeComponent();
            _productLogic = productLogic;
            KeyPreview = true;
        }

        private void FormProductsList_Load(object sender, EventArgs e)
        {
            var idColumn = new DataGridViewColumn();
            var nameColumn = new DataGridViewColumn();
            var dataGridViewCell = new DataGridViewTextBoxCell();
            idColumn.Visible = false;
            idColumn.DataPropertyName = "Id";
            idColumn.Name = "Id";
            idColumn.CellTemplate = dataGridViewCell;
            nameColumn.DataPropertyName = "Name";
            nameColumn.HeaderText = "Название";
            nameColumn.Name = "Name";
            nameColumn.CellTemplate = dataGridViewCell;
            dataGridView.Columns.Add(idColumn);
            dataGridView.Columns.Add(nameColumn);
            LoadData();
        }

        private void LoadData()
        {
            try
            {
                _products = _productLogic.ReadList();
                dataGridView.Rows.Clear();
                foreach (var product in _products)
                {
                    dataGridView.Rows.Add(product);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("Произошла ошибка при загрузке данных:\n" + ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void FormProductsList_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Insert)
            {
                dataGridView.Rows.Add();
                dataGridView.CurrentCell = dataGridView.Rows[dataGridView.RowCount - 1].Cells[1];
                e.Handled = true;
            }
            else if (e.KeyCode == Keys.Delete)
            {
                if (dataGridView.SelectedRows.Count == 1)
                {
                    DataGridViewRow selectedRow = dataGridView.SelectedRows[0];
                    var selectedItemIdValue = selectedRow.Cells["Id"].Value;
                    if (selectedItemIdValue == null)
                    {
                        dataGridView.Rows.Remove(selectedRow);
                        return;
                    }
                    int selectedItemId = Convert.ToInt32(selectedItemIdValue);
                    string selectedItemName = (string)selectedRow.Cells["Name"].Value;
                    var result = MessageBox.Show(
                        $"Удалить товар \"{selectedRow.Cells["Id"].Value}\"?",
                        "Подтверждение",
                        MessageBoxButtons.YesNo,
                        MessageBoxIcon.Question
                    );

                    if (result == DialogResult.Yes)
                    {
                        if (_productLogic.Delete(new ProductSearchModel { Id = selectedItemId }))
                        {
                            MessageBox.Show($"Товар \"{selectedItemName}\" удален");
                            LoadData();
                        }
                        else
                        {
                            MessageBox.Show("Ошибка удаления", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        }
                    }
                }
            }
        }

        private void dataGridView_CellEndEdit(object sender, DataGridViewCellEventArgs e)
        {
            try
            {
                DataGridViewRow currentRow = dataGridView.Rows[e.RowIndex];
                if (currentRow.Cells["Id"].Value == null || Convert.ToInt32(currentRow.Cells["Id"].Value) == 0)
                {
                    _productLogic.Create(new ProductBindingModel
                    {
                        Name = currentRow.Cells["Name"].Value.ToString()!,
                    });
                }
                else
                {
                    _productLogic.Update(new ProductBindingModel
                    {
                        Id = Convert.ToInt32(currentRow.Cells["Id"].Value),
                        Name = currentRow.Cells["Name"].Value.ToString(),
                    });
                }
                MessageBox.Show("Запись сохранена");
            }
            catch (Exception ex)
            {
                MessageBox.Show("Ошибка сохранения:\n" + ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void dataGridView_CellValidating(object sender, DataGridViewCellValidatingEventArgs e)
        {
            string? userInput = e.FormattedValue.ToString();

            if (string.IsNullOrWhiteSpace(userInput))
            {
                MessageBox.Show("Нельзя сохранить пустую строку", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                e.Cancel = true;
            }
        }
    }
}
